@@ -144,7 +144,7 @@ cache_rx = '_[abcdef0123456789]{32}[.](rdb|rdx|RData)$'
 #'   files \file{__objects} and \file{__globals}.
 #' @export
 #' @seealso \code{\link{dep_prev}}
-#' @references \url{http://yihui.name/knitr/demo/cache/}
+#' @references \url{https://yihui.name/knitr/demo/cache/}
 dep_auto = function(path = opts_chunk$get('cache.path')) {
   # this function should be evaluated in the original working directory
   owd = setwd(opts_knit$get('output.dir')); on.exit(setwd(owd))
@@ -243,7 +243,7 @@ load_cache = function(
 #'   effect.
 #' @export
 #' @seealso \code{\link{dep_auto}}
-#' @references \url{http://yihui.name/knitr/demo/cache/}
+#' @references \url{https://yihui.name/knitr/demo/cache/}
 dep_prev = function() {
   labs = names(knit_code$get())
   if ((n <- length(labs)) < 2L) return() # one chunk or less; no sense of deps
@@ -263,7 +263,7 @@ dep_prev = function() {
 #' references.
 #' @export
 #' @format NULL
-#' @references \url{http://yihui.name/knitr/demo/cache/}
+#' @references \url{https://yihui.name/knitr/demo/cache/}
 #' @examples eval(rand_seed)
 #' rnorm(1) # .Random.seed is created (or modified)
 #' eval(rand_seed)
@@ -283,9 +283,19 @@ rand_seed = quote({
 #'   multiple documents share the same cache directory. You are recommended to
 #'   call \code{clean_cache(FALSE)} and carefully check the list of files (if
 #'   any) before you really delete them (\code{clean_cache(TRUE)}).
+#'
+#'   This function must be called within a code chunk in a source document,
+#'   since it needs to know all chunk labels of the current document to
+#'   determine which labels are no longer present, and delete cache
+#'   corresponding to these labels.
 #' @export
 clean_cache = function(clean = FALSE, path = opts_chunk$get('cache.path')) {
-  owd = setwd(opts_knit$get('output.dir')); on.exit(setwd(owd))
+  odir = opts_knit$get('output.dir')
+  if (is.null(odir)) {
+    warning('This function must be called inside a source document')
+    return()
+  }
+  owd = setwd(odir); on.exit(setwd(owd))
   if (file_test('-d', path)) {
     p0 = path; p1 = ''
   } else {
