@@ -3,7 +3,9 @@
 #' This function takes an input file, extracts the R code in it according to a
 #' list of patterns, evaluates the code and writes the output in another file.
 #' It can also tangle R source code from the input document (\code{purl()} is a
-#' wrapper to \code{knit(..., tangle = TRUE)}).
+#' wrapper to \code{knit(..., tangle = TRUE)}). The \code{knitr.purl.inline}
+#' option can be used to also tangle the code of inline expressions (disabled
+#' by default).
 #'
 #' For most of the time, it is not necessary to set any options outside the
 #' input document; in other words, a single call like
@@ -111,9 +113,9 @@
 #'   the R script generated by \code{purl()}. This seems to be obvious, but some
 #'   people \href{http://bit.ly/SnLi6h}{just do not get it}.
 #' @export
-#' @references Package homepage: \url{http://yihui.name/knitr/}. The \pkg{knitr}
-#'   \href{http://yihui.name/knitr/demo/manual/}{main manual}: and
-#'   \href{http://yihui.name/knitr/demo/graphics}{graphics manual}.
+#' @references Package homepage: \url{https://yihui.name/knitr/}. The \pkg{knitr}
+#'   \href{https://yihui.name/knitr/demo/manual/}{main manual}: and
+#'   \href{https://yihui.name/knitr/demo/graphics/}{graphics manual}.
 #'
 #'   See \code{citation('knitr')} for the citation information.
 #' @examples library(knitr)
@@ -159,6 +161,7 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     # restore chunk options after parent exits
     optc = opts_chunk$get(); on.exit(opts_chunk$restore(optc), add = TRUE)
     ocode = knit_code$get(); on.exit(knit_code$restore(ocode), add = TRUE)
+    on.exit(opts_current$restore(), add = TRUE)
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
     opts_knit$set(tangle = tangle, encoding = encoding,
                   progress = opts_knit$get('progress') && !quiet
@@ -374,7 +377,7 @@ auto_format = function(ext) {
 #' @return A character string of the content of the compiled child document is
 #'   returned as a character string so it can be written back to the parent
 #'   document directly.
-#' @references \url{http://yihui.name/knitr/demo/child/}
+#' @references \url{https://yihui.name/knitr/demo/child/}
 #' @note This function is not supposed be called directly like
 #'   \code{\link{knit}()}; instead it must be placed in a parent document to let
 #'   \code{\link{knit}()} call it indirectly.
@@ -551,7 +554,7 @@ wrap.recordedplot = function(x, options) {
     MoreArgs = list(plot = x, name = name, options = options), SIMPLIFY = FALSE
   )[[1]]
   if (options$fig.show == 'hide') return('')
-  knit_hooks$get('plot')(file, reduce_plot_opts(options))
+  in_base_dir(knit_hooks$get('plot')(file, reduce_plot_opts(options)))
 }
 
 #' @export
